(function(window, document, undefined){
	var utility = {
		KEY : {
			ESC: 27,
			TAB: 9,
			ENTER: 13
		},
		support: {
			dataset: !!document.body.dataset,
			notification: false
		},
		notifyObj : null,
		timerObj : null,
		notify : function (msg, timeout) {
			this.triggerCustomEvent('sogouExplorerExtensionNotify', {
				message : msg,
				timeout : timeout
			});
		},
		closeNotify : function () {
			this.triggerCustomEvent('sogouExplorerExtensionCloseNotify');
		},
		setPref : function (name, value) {
			window.localStorage.setItem(name, value);
		},
		getPref : function (name) {
			return window.localStorage[name];
		},
		getTimeInfo : function () {
			var d = new Date();
			return d.getHours() + ":" + (d.getMinutes() < 10 ? "0" : "") + d.getMinutes() + ":" + (d.getSeconds() < 10 ? "0" : "") + d.getSeconds();
		},
		savePrefs : function (obj, prefix) {
			var objs = obj.find("input");
			var self = this;
			
			objs.change(function () {
				var type = this.getAttribute("type");
				if (type == "text")
					self.setPref(prefix + "_" + this.getAttribute("id"), $(this).val());
				else if (type == "checkbox")
					self.setPref(prefix + "_" + this.getAttribute("id"), this.checked ? 1 : 0);
			})
		},
		reloadPrefs : function (obj, prefix) {
			var objs = obj.find("input");
			var self = this;
			
			objs.each(function () {
				var e = $(this);
				var type = e.attr("type");
				var id = e.attr("id");
				
				var value = self.getPref(prefix + "_" + id);
				if (typeof(value) == "undefined")
					return;
				
				if (type == "text")
					e.val(value);
				else if (type == "checkbox")
					this.checked = value == "1";
			});
			this.savePrefs(obj, prefix);
		},
		getErrorMsg : function (msg) {
			var m = msg.match(/var\s+message\s*=\s*"([^"]*)/);
			return m && m[1] ? m[1] : "&lt;未知信息&gt;";
		},
		tip: {
			showMask: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionTip.showMask');
			},
			hide: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionTip.hide');
			},
			neterror: {
				popup: function(errorCount, callback){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.neterror', {
						errorCount: errorCount,
						callback: callback
					});
				}
			},
			wait: {
				popup: function(count, callback){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.wait', {
						count: count,
						callback: callback
					});
				}
			},
			noUsername: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noUsername');
				}
			},
			username: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.username');
				}
			},
			noPwd: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noPwd');
				}
			},
			pwd: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.pwd');
				}
			},
			noValidateCode: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noValidateCode');
				}
			},
			validateCode: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.validateCode');
				}
			},
			deny: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.deny');
				}
			},
			
			noFirstTicketTime: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noFirstTicketTime');
				}
			},
			noFirstTicketDate: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noFirstTicketDate');
				}
			},
			
			disconnected: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.disconnected');
				}
			}
		},
		queryNotificationSupport: function(){
			utility.triggerCustomEvent('sogouExplorerExtensionNotification.querySupport');
		},
		setNotificationSupport: function(bool){
			utility.support.notification = bool;
		},
		triggerCustomEvent: function(type, data){
			var event;
			if (typeof window.CustomEvent === 'function') {
				event = new window.CustomEvent(type, {
					detail: data ? data : {} 
				});
				document.dispatchEvent(event);
			}else if(typeof document.createEvent === 'function'){
				event = document.createEvent('Event');
				event.initEvent(type);
				
				if(data){
					for(var key in data){
						document.body.dataset[type + '.' + key] = data[key];
					}
				}
				document.dispatchEvent(event);
			}
		},
		pingback: {
			begin: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.begin');
			},
			startlogin: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.startlogin');
			},
			loginsuc: function(autologin_count){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.loginsuc', {count: autologin_count});
			},
			startbuy: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.startbuy');
			},
			buysuc: function(autorefresh_count){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.buysuc', {count: autorefresh_count});
			},
			starttimer: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.starttimer');
			},
			timersuc: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.timersuc');
			}
		}
	};

	var console = window.console || {log: window.alert};
	
	document.addEventListener("sogouExplorerExtensionNotification.querySupportOver", function(e){var detail = e.detail || {support: document.body.dataset["sogouExplorerExtenionNotification.querySupportOver.support"]}; utility.setNotificationSupport(detail.support);}, false);
	utility.queryNotificationSupport();
	
	var initConfirmOrder = function(){
		function delayInvoke(callback, timeout, no_tip) {
			var is_show = (no_tip == undefined ? true:false);
			var e = $("#countEle");
			if (e.length == 0) is_show = false;

			if (timeout <= 0) {
			  if(is_show) e.html("");
			  callback();
			} else {
			  if (is_show) {
				var str = (Math.floor(timeout / 100) / 10) + '';
				if (str.indexOf(".") == -1) str += ".0";
				e.html(str + " 秒后开始");
			  }
			  setTimeout(function () {
				delayInvoke(callback, timeout - 500);
			  }, 500);
			}
		}
	
		var g_auto_commit_counter = 0;
		var randCode = "";
		var g_is_submitting = false;
		var tourFlag = 'dc';
		var randEl = $("#rand");

		if ($(".error_text").length > 0 && parent.$("#orderForm").length > 0) {
		  parent.resubmitForm();

		  return;
		}

		(function () {
		  /'(dc|fc|wc|gc)'/.exec($("div.tj_btn :button:eq(2)")[0].onclick + '');
		  tourFlag = RegExp.$1;
		})();

		function setCommitButtonText() {
		  //if (g_is_submitting) $("#btnAutoSubmit").html("提交中，点击停止...");
		  //else $("#btnAutoSubmit").html("自动提交");
		  
			if(g_is_submitting){
				//$('#btnAutoSubmit').removeClass('stopped').addClass('running');
				$('#btnAutoSubmit').html('停止自动提交');
			}else{
				//$('#btnAutoSubmit').removeClass('running').addClass('stopped');
				$('#btnAutoSubmit').html('自动提交订单');
			}
		}

		// stop the auto commit process!
		function stop(msg) {
		  g_is_submitting = false;

		  setCommitButtonText();

		  setCurOperationInfo(true, "自动提交已停止");
		  setTipMessage(msg);

		  $('#coutEle').html('');

		  $("div.tj_btn button, div.tj_btn input").each(function () {
			this.disabled = false;
			$(this).removeClass().addClass("long_button_u");
		  });
		}

		var reloadCode = function () {
		  $("#img_rrand_code").click();
		  $("#rand").val("")[0].select();
		};

		//订单等待时间过久的警告
		var waitTimeTooLong_alert = false;

		var g_test_only = false;

		function submitForm() {
		  setCurOperationInfo(true, "正在提交订单...");
		  if (g_is_submitting === false) {
			stop("自动提交已经停止，请尝试手工“提交订单”");
			return;
		  }

		  randEl[0].blur();
		  stopCheckCount();
		  if (!window.submit_form_check || !submit_form_check("confirmPassenger")) {
			stop("请填写完整表单");
			return;
		  }
			
		  ++g_auto_commit_counter;
		  waitTimeTooLong_alert = false;

		  if (g_test_only === true) {
			delayInvoke(submitForm, 1000);
			return;
		  }
		
		  $("#confirmPassenger").ajaxSubmit({
			url: 'confirmPassengerAction.do?method=checkOrderInfo&rand=' + $("#rand").val(),
			type: "POST",
			data: {
			  tFlag: tourFlag
			},
			dataType: "json",
			timeout: 8000,
			success: function (data) {
			  if ('Y' != data.errMsg || 'N' == data.checkHuimd || 'N' == data.check608) {
				stop(data.msg || data.errMsg);
				reloadCode();
			  } else {
				delayInvoke(ajaxQueryQueueCount, 3000);
			  }
			},
			error: function (msg) {
			  setTipMessage("自动订单提交验证时出错，正在自动重提...");
			  delayInvoke(submitForm, 2000);
			}
		  });
		}

		function ajaxQueryQueueCount() {
		  setCurOperationInfo(true, "正在自动查询排队人数....");
		  if (g_is_submitting === false) {
			stop("自动提交已经停止，请尝试手工“提交订单”");
			return;
		  }
		  ++g_auto_commit_counter;
		  var queryLeftData = {
			train_date: $("#start_date").val(),
			train_no: $("#train_no").val(),
			station: $("#station_train_code").val(),
			seat: $("#passenger_1_seat").val(),
			from: $("#from_station_telecode").val(),
			to: $("#to_station_telecode").val(),
			ticket: $("#left_ticket").val()
		  };
		  $.ajax({
			url: '/otsweb/order/confirmPassengerAction.do?method=getQueueCount', //&_=' + Math.random(),
			data: queryLeftData,
			type: 'GET',
			timeout: 10000,
			dataType: 'json',
			success: function (result) {
			  if (result.op_2) {
				var tip, delay_msec;
				if (result.count == 0) {
				  tip = "尚有余票，目前排队人数已超过余票张数，";
				  delay_msec = 5000;
				} else {
				  tip = "排队人数（" + result.count + "）过多，";
				  delay_msec = 2000;
				}
				setTipMessage( tip + "铁道部拒绝排队，正在努力重试，如果长时间失败请重新订票或尝试手工提交！");
				delayInvoke(ajaxQueryQueueCount, delay_msec);
				return;
			  }
			  submitConfirmOrder();
			},
			error: function () {
			  delayInvoke(ajaxQueryQueueCount, 2000);
			}
		  });
		}
		
		function submitConfirmOrder() {
		  setCurOperationInfo(true, "正在自动排队....");
		  if (g_is_submitting === false) {
			stop("自动提交已经停止，请尝试手工“提交订单”");
			return;
		  }
		  ++g_auto_commit_counter;
		  //异步下单
		  var order_type = 'confirmSingleForQueue'; //'dc' 单程
		  if (tourFlag == 'wc') {
			// 异步下单-往程
			order_type = 'confirmPassengerInfoGoForQueue';
		  } else if (tourFlag == 'fc') {
			// 异步下单-返程
			order_type = 'confirmPassengerInfoBackForQueue';
		  } else if (tourFlag == 'gc') {
			// 异步下单-改签
			order_type = 'confirmPassengerInfoResignForQueue';
		  }
		  $.ajax({
			url: '/otsweb/order/confirmPassengerAction.do?method=' + order_type,
			data: $('#confirmPassenger').serialize(),
			type: "POST",
			timeout: 10000,
			dataType: 'json',
			success: function (msg) {
			  console.log(msg);
			  var errmsg = msg.errMsg;
			  if (errmsg != 'Y') {
				if (errmsg.indexOf("包含未付款订单") != -1) {
				  stop("包含未付款订单");
				  alert("您有未支付订单! 赶紧点“确定”去支付或取消了，否则无法继续提交订单。");
				  window.location.replace("/otsweb/order/myOrderAction.do?method=queryMyOrderNotComplete&leftmenu=Y");
				  return;
				}
				if (errmsg.indexOf("重复提交") != -1) {
				  stop("重复提交错误，已在自动更新令牌，需要您重新输入验证码（若多次仍无法提交订单，请重新订票或尝试手工提交！）");
				  reloadToken();
				  reloadCode();
				  return;
				}
				if (errmsg.indexOf("后台处理异常") != -1 || errmsg.indexOf("非法请求") != -1) {
				  stop("后台处理异常，请返回查询页重新订票或尝试手工提交！！");
				  return;
				}
				if (errmsg.indexOf("包含排队中") != -1) {
				  waitingForQueueComplete();
				  return;
				}

				stop(errmsg);
				reloadCode();
			  } else {
				waitingForQueueComplete();
			  }
			},
			error: function (msg) {
			  setTipMessage("自动请求排队出错，正在继续自动重试...");
			  delayInvoke(submitForm, 3000);
			}
		  });
		}

		function reloadToken(submit) {
		  setTipMessage("正在自动更新令牌....");
		  $.ajax({
			url: "/otsweb/order/confirmPassengerAction.do?method=init",
			data: null,
			timeout: 10000,
			type: "POST",
			dataType: "text",
			success: function (text) {
			  if (!/TOKEN"\s*value="([a-f\d]+)"/i.test(text)) {
				setTipMessage("更新令牌失败，正在继续自动重试...");
				delayInvoke(reloadToken, 1000);
			  } else {
				var token = RegExp.$1;
				setTipMessage("令牌更新完成，如果仍然无法自动提交或手工提交，请重新订票！");
				$("input[name=org.apache.struts.taglib.html.TOKEN]").val(token);
			  }
			},
			error: function () {
			  setTipMessage("更新令牌出错，正在继续自动重试...");
			  delayInvoke(reloadToken, 3000);
			}
		  });
		}

		function getSecondInfo(second) {
		  var show_time = "";
		  var hour = parseInt(second / 3600); //时
		  if (hour > 0) {
			show_time = hour + "小时";
			second = second % 3600;
		  }
		  var minute = parseInt(second / 60); //分
		  if (minute >= 1) {
			show_time = show_time + minute + "分";
			second = second % 60;
		  } else if (hour >= 1 && second > 0) {
			show_time = show_time + "0分";
		  }
		  if (second > 0) {
			show_time = show_time + second + "秒";
		  }

		  return show_time;
		}

		function waitingForQueueComplete() {
		  setCurOperationInfo(true, "订单成功排队, 正在自动等待排队完成...");
		  if (g_is_submitting === false) {
			stop("自动提交已经停止，请尝试手工“提交订单”");
			return;
		  }
		  ++g_auto_commit_counter;

		  $.ajax({
			url: '/otsweb/order/myOrderAction.do?method=queryOrderWaitTime&tourFlag=' + tourFlag,// + '&' + Math.random(),
			data: {},
			type: 'GET',
			timeout: 10000,
			dataType: 'json',
			success: function (json) {
			  if (json.waitTime == -1 || json.waitTime == 0) {
				//TODO：声音：赶紧去支付！
				utility.notify("订票成功，可以付款了！");
				if (json.orderId) window.location.replace("/otsweb/order/confirmPassengerAction.do?method=payOrder&orderSequence_no=" + json.orderId);
				else window.location.replace('/otsweb/order/myOrderAction.do?method=queryMyOrderNotComplete&leftmenu=Y');
			  } else if (json.waitTime == -3) {
				var msg = "很抱歉, 您的本次订单提交已被铁道部取消，请重新订票或尝试手工提交！";
				utility.notify(msg);
				stop(msg);
				reloadCode();
			  } else if (json.waitTime == -2) {
				var msg = "很抱歉, 您的本次订单提交排队失败：" + json.msg + ', 请重新订票或尝试手工提交！';
				reloadToken();
				utility.notify(msg);
				stop(msg);
				reloadCode();
			  } else if (json.waitTime < 0) {
				var msg = '很抱歉, 未知的状态信息 : waitTime=' + json.waitTime + ', 可能已成功，请验证未支付订单.';
				setTipMessage(msg);
				utility.notify(msg);
				location.href = '/otsweb/order/myOrderAction.do?method=queryMyOrderNotComplete&leftmenu=Y';
			  } else {
				var msg = "订单需要 " + getSecondInfo(json.waitTime) + " 处理完成， 请等待。（排队人数=" + (json.waitCount || "未知") + "）";
				if (json.waitTime > 1800) {
				  msg += "<span style='color:red; font-weight: bold;'>提示：排队时间大于30分钟，成功率较低，请尽快电话订票或重新订票！</span>";
				}
				setTipMessage(msg);

				if (json.waitTime > 1800 && !waitTimeTooLong_alert) {
				  waitTimeTooLong_alert = true;
				  utility.notify("提示：排队时间大于30分钟，成功率较低，请尽快电话订票或重新订票！");
				}
				var delay_secs = 2000 + 1000 * Math.round(json.waitTime / 60);
				delayInvoke(waitingForQueueComplete, delay_secs);
			  }
			},
			error: function (json) {
			  utility.notify("请求发生异常，可能是登录状态不对，如果登录状态没有问题，请手动进入“未完成订单”页面查询。");
			  self.location.reload();
			}
		  });
		}
		
		$('#confirmPassenger .tj_btn').append(
			$("<span style='cursor: pointer;margin-left: 10px;padding: 4px 6px;font-size: 125%; background: #2CC03E;border-color: #259A33;border-right-color: #2CC03E;border-bottom-color:#2CC03E;color: white;border-radius: 5px;text-shadow: -1px -1px 0 rgba(0, 0, 0, 0.2);'></span>")
			.attr("id", "btnAutoSubmit")
			.html("自动提交订单")
			.click(function () {
			  if (g_is_submitting) {
				stop("您主动停止了自动提交！");
				//document.getElementById("autoStartCommit").checked = false;
				$("#autoStartCommit").attr("checked", true).click().attr("checked", false);
			  } else {
				g_auto_commit_counter = 0;
				g_is_submitting = true;
				$("#tipScript").show();
				setCommitButtonText();
				submitForm();
			  }
			})
		);
		//.append(
		//$("<span>").html("&nbsp;(帮您自动确认订单并在允许的情况下自动查询订单是否排队成功）")));

		var KEY = utility.KEY;
		
		randEl.keyup(function (e) {
		  //if (!document.getElementById("autoStartCommit").checked) return;
		  //if (e.charCode == 13 || (randEl.val().length == 4)) {
		 
		  if (e.keyCode === KEY.ENTER) {
			$("#btnAutoSubmit").click();
		  }
		});

		//如果只有一个联系人，自动勾上，但是千万别学别人去保持用户信息！
		var startAutoSelectTime = +new Date,
		retryTimes = 0;
		window.setTimeout(function () {
		  retryTimes++;
		  var contacts = $('#passenger_single_tb_id td[colspan] input');
		  if (contacts.length) {
			if (contacts.length == 1) {
			  contacts[0].click();
			}
		  } else {
			if ((+new Date - startAutoSelectTime) < 10000) {
			  setTimeout(arguments.callee, 100 * retryTimes);
			}
		  }
		}, 100);

		//清除上次保存的预定信息
		var lastform = null;
		if (parent) {
		  lastform = parent.$("#orderForm");
		  if (lastform) lastform.attr("success", "1");
		}

		//输入完即提交
		//$(".table_qr tr:last").before("<tr><td colspan='9'><label style='color:blue;'><input type='checkbox' id='autoStartCommit' /> 输入验证码后立刻开始自动提交，如果希望手工提交，请不要勾选。</label></td></tr>");
		//document.getElementById("autoStartCommit").checked = typeof (window.localStorage["ll_disableAutoStartCommit"]) == 'undefined';
		
		/*
		$("#autoStartCommit").change(function () {
		  if (this.checked) window.localStorage.removeItem("ll_disableAutoStartCommit");
		  else window.localStorage.setItem("ll_disableAutoStartCommit", "1");
		});*/

		//进度提示框
		$("table.table_qr tr:last").before("<tr><td style='border-top:1px dotted #ccc;height:100px; display:none;' colspan='9' id='orderCountCell'></td></tr><tr><td style='border-top:1px dotted #ccc;' colspan='9'><ul id='tipScript' style='display:none;'>" +
		  "<li><span id='opStatusEle' style='font-weight:bold; color:#EA5200;'>等待操作</span><span id='countEle' style='padding-left:10px;'></span> </li>" +
		  "<li style='color:#EA5200;'><strong>提示信息</strong>：<span>--</span></li>" +
		  "<li style='color:black;'><strong>提示时间</strong>：<span>--</span></li></ul></td></tr>");

		var ele_tip_msg_ = $("#tipScript li");
		var ele_cur_op_ = $("#opStatusEle");

		// show operation detail: running is useless
		function setCurOperationInfo(no_use, msg) {
		  ele_cur_op_.html("第 " + g_auto_commit_counter + " 次操作：" + msg);
		  if (g_auto_commit_counter > 15) {
			g_is_submitting = false;
			//$("#autoStartCommit").attr("checked", true).click().attr("checked", false);
			//自动转为手工提交订单
			//$(".tj_btn button:eq(2)").trigger("click");
		  }
		  setTipMessage("----");
		}

		function getTimeInfo() {
		  var d = new Date();
		  return d.getHours() + ":" + (d.getMinutes() < 10 ? "0" : "") + d.getMinutes() + ":" + (d.getSeconds() < 10 ? "0" : "") + d.getSeconds();
		}

		// show error message detail
		function setTipMessage(msg) {
		  ele_tip_msg_.eq(2).find("span").html(getTimeInfo());
		  ele_tip_msg_.eq(1).find("span").html(msg);
		}

		$(".table_qr tr:last").before("<tr><td colspan='9'><label><input type='checkbox' id='showHelp' /> 显示帮助 </label></td></tr>");
		document.getElementById("showHelp").checked = typeof (window.localStorage["showHelp"]) != 'undefined';
		$("#showHelp").change(function () {
		  if (this.checked) {
			window.localStorage.setItem("showHelp", "1");
			$("table.table_qr tr:last").show();
		  } else {
			window.localStorage.removeItem("showHelp");
			$("table.table_qr tr:last").hide();
		  }
		}).change();

		//#region “票况查询”
		var stopCheckCount = null;
		var gUrlQueryLeftTicket = "querySingleAction.do?method=queryLeftTicket";
		var gurlGetQueueCount = "confirmPassengerAction.do?method=getQueueCount";

		(function () {
		  var data = {
			train_date: $("#start_date").val(),
			station: $("#station_train_code").val(),
			seat: "",
			from: $("#from_station_telecode").val(),
			to: $("#to_station_telecode").val(),
			ticket: $("#left_ticket").val()
		  };

		  var allSeats = $("#passenger_1_seat option");
		  var queue = [];
		  var checkCountStopped = true; //禁用余票查询功能！

		  function beginCheck() {
			if (checkCountStopped) return;

			$("#orderCountCell").show();
			var html = [];
			html.push("<span id='leftTicketTipMsg' style='font-size:125%;color:red;'>本次列车余票最新排队情况（每2秒刷新一次直至您开始提交订单）：</span>");


			allSeats.each(function () {
			  queue.push({
				id: this.value,
				name: this.text
			  });
			  html.push("【<span style='color:blue; font-weight: bold;'>" + this.text + "</span>】<span id='queueStatus_" + this.value + "'>等待查询中....</span>");
			});
			$("#orderCountCell").html(html.join("<br />"));
			if (queue.length > 0) executeQueue();
		  }

		  function checkTicketAvailable() {
			var queryLeftData = {
			  'orderRequest.train_date': $('#start_date').val(),
			  'orderRequest.from_station_telecode': $('#from_station_telecode').val(),
			  'orderRequest.to_station_telecode': $('#to_station_telecode').val(),
			  'orderRequest.train_no': $('#train_no').val(),
			  'trainPassType': 'QB',
			  'trainClass': 'QB#D#Z#T#K#QT#',
			  'includeStudent': 00,
			  'seatTypeAndNum': '',
			  'orderRequest.start_time_str': '00:00--24:00'
			};
			$.ajax({
			  url: gUrlQueryLeftTicket,
			  data: queryLeftData,
			  timeout: 10000,
			  type: "GET",
			  dataType: "text",
			  success: function (text) {
				window.ticketAvailable = '';
				if (/(([\da-zA-Z]\*{5,5}\d{4,4})+)/gi.test(text)) {
				  window.ticketAvailable = RegExp.$1;
				}
			  },
			  error: function () {}
			});
		  }

		  function executeQueue() {
			if (checkCountStopped) return;

			var type = queue.shift();
			queue.push(type);

			data.seat = type.id;
			var strLeftTicket = '';
			checkTicketAvailable();
			if (window.ticketAvailable) {
			  strLeftTicket = window.ticketAvailable;
			}
			$.ajax({
			  url: gurlGetQueueCount,
			  data: data,
			  timeout: 10000,
			  type: "GET",
			  dataType: "json",
			  success: function (data) {
				var msg = "余票查询结果未识别";
				try {
				  var msg = "余票：<strong>" + getTicketCountDesc(strLeftTicket, type.id) + "</strong>";
				  msg += "，当前【<span style='color:blue; font-weight: bold;'>" + data.count + "</span>】人排队订票，";
				  if (data.op_2) { // || data.count > strLeftTicket) {
					msg += "<span style='color:blue; font-weight: red;'>请您尽快提交</span>。";
				  } else {
					msg += "请您抓紧提交";
				  }
				  msg += "&nbsp;&nbsp;&nbsp;&nbsp;（更新于：" + getTimeInfo() + "）";
				} catch (e) {}
				$("#queueStatus_" + type.id).html(msg);
				setTimeout(executeQueue, 2000);
			  },
			  error: function () {
				setTimeout(executeQueue, 3000);
			  }
			});
		  }

		  stopCheckCount = function () {
			checkCountStopped = true;
			//$("#orderCountCell").hide();//这部分的hide会导致“手工提交”按钮位置变动，影响操作
			$('#leftTicketTipMsg').html("本次列车余票排队情况（最后更新时间：" + getTimeInfo() + "）");
		  }

		  // 停止“票况查询” 
		  $(".tj_btn button:eq(2)").bind("mousedown", function () {
			stopCheckCount();
		  });

		  // beginCheck(); //禁用票况查询
		})();

		//#endregion

		(function () {
		  var obj = document.getElementById("rand");

		  var oldOnload = window.onload;
		  window.onload = function () {
			if (oldOnload) oldOnload();
			obj.select();
		  };
		  obj.select();
		})();

		parent.$("#main").css("height", ($(document).height() + 300) + "px");
		parent.window.setHeight(parent.window);
	

		function submit() {
			//Auto select the first user when not selected
			if (!$("input._checkbox_class:checked").length) {
			  try {
				//Will failed in IE
				$("input._checkbox_class:first").click();
			  } catch (e) {};
			}
			//passengerTickets

			var userInfoUrl = getHostCtx() + '/order/myOrderAction.do?method=queryMyOrderNotComplete&leftmenu=Y';

			var count = 1,
			  freq = 1000,
			  doing = false,
			  timer, $msg = $("<div style='padding-left:470px;'></div>");

			function submitForm() {
			  timer = null;
			  //更改提交列车日期参数
			  //var wantDate = $("#startdatepicker").val();
			  //$("#start_date").val(wantDate);
			  //$("#_train_date_str").val(wantDate);

			  jQuery.ajax({
				url: $("#confirmPassenger").attr('action'),
				data: $('#confirmPassenger').serialize(),
				beforeSend: function (xhr) {
				  try {
					xhr.setRequestHeader('X-Requested-With', {
					  toString: function () {
						return '';
					  }
					});
					xhr.setRequestHeader('Cache-Control', 'max-age=0');
					xhr.setRequestHeader('Accept', 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8');
				  } catch (e) {};
				},
				type: "POST",
				timeout: 30000,
				success: function (msg) {
				  //Refresh token
				  var match = msg && msg.match(/org\.apache\.struts\.taglib\.html\.TOKEN['"]?\s*value=['"]?([^'">]+)/i);
				  var newToken = match && match[1];
				  if (newToken) {
					$("input[name='org.apache.struts.taglib.html.TOKEN']").val(newToken);
				  }

				  if (msg.indexOf('payButton') > -1) {
					//Success!
					
					utility.notify("恭喜，车票预订成！");
					setTimeout(function () {
					  if (confirm("车票预订成，去付款？")) {
						window.location.replace(userInfoUrl);
					  } else {
						if (audio && !audio.paused) audio.pause();
					  }
					}, 100);
					return;
				  } else if (msg.indexOf('未处理的订单') > -1) {
					utility.notify("有未处理的订单!");
					window.location.replace(userInfoUrl);
					return;
				  }
				  var reTryMessage = [
					'用户过多', '确认客票的状态后再尝试后续操作', '请不要重复提交', '没有足够的票!', '车次不开行'];
				  for (var i = reTryMessage.length - 1; i >= 0; i--) {
					if (msg.indexOf(reTryMessage[i]) > -1) {
					  reSubmitForm(reTryMessage[i]);
					  return;
					}
				  }
				  // 铁道部修改验证码规则后  update by 冯岩
				  if (msg.indexOf("输入的验证码不正确") > -1) {
					$("#img_rrand_code").click();
					$("#rand").focus().select();
					stop();
					return;
				  }
				  //Parse error message
				  msg = msg.match(/var\s+message\s*=\s*"([^"]*)/);
				  stop(msg && msg[1] || '出错了。。。。 啥错？ 我也不知道。。。。。');
				},
				error: function (msg) {
				  reSubmitForm("网络错误");
				}
			  });
			};

			function reSubmitForm(msg) {
			  if (!doing) return;
			  count++;
			  $msg.html("(" + count + ")次自动提交中... " + (msg || ""));
			  timer = setTimeout(submitForm, freq || 50);
			}

			function stop(msg) {
			  doing = false;
			  $msg.html("(" + count + ")次 已停止");
			  $('#refreshButton').html("自动提交订单");
			  timer && clearTimeout(timer);
			  msg && alert(msg);
			}

			function reloadSeat() {
			  $("select[name$='_seat']").html('<option value="M" selected="">一等座</option><option value="O" selected="">二等座</option><option value="1">硬座</option><option value="3">硬卧</option><option value="4">软卧</option>');
			}
			//初始化

			if ($("#refreshButton").size() < 1) {

			  //  //重置后加载所有席别
			  //  $("select[name$='_seat']").each(function(){this.blur(function(){
			  //    alert(this.attr("id") + "blur");
			  //  })});
			  ////初始化所有席别
			  //$(".qr_box :checkbox[name^='checkbox']").each(function(){$(this).click(reloadSeat)});
			  //reloadSeat();

			  //日期可选
			  $("td.bluetext:first").html('<input type="text" name="orderRequest.train_date" value="' + $("#start_date").val() + '" id="startdatepicker" style="width: 150px;" class="input_20txt"  onfocus="WdatePicker({firstDayOfWeek:1})" />');
			  $("#start_date").remove();

			  $(".tj_btn").append($("<a style='padding: 5px 10px; background: #2CC03E;border-color: #259A33;border-right-color: #2CC03E;border-bottom-color:#2CC03E;color: white;border-radius: 5px;text-shadow: -1px -1px 0 rgba(0, 0, 0, 0.2);'></a>").attr("id", "refreshButton").html("自动提交订单").click(function () {
				//alert('开始自动提交订单，请点确定后耐心等待！');
				if (this.innerHTML.indexOf("自动提交订单") == -1) {
				  //doing
				  stop();
				} else {
				  if (window.submit_form_check && !window.submit_form_check("confirmPassenger")) {
					return;
				  }
				  count = 0;
				  doing = true;
				  this.innerHTML = "停止自动提交";
				  reSubmitForm();
				}
				return false;
			  }));
			  $(".tj_btn").append("自动提交频率：")
				.append($("<select id='freq'><option value='50' >频繁</option><option value='500' selected='' >正常</option><option value='2000' >缓慢</option></select>").change(function () {
				freq = parseInt($(this).val());
			  }))
				.append($msg);
			  //alert('如果使用自动提交订单功能，请在确认订单正确无误后，再点击自动提交按钮！');

			  //铁道路修改验证码规则后 优化 by 冯岩
			  $("#rand").bind('keydown', function (e) {
				var key = e.which;
				if (key == 13) {
				  $("#refreshButton").click();
				}
			  });
			}
		}
		
	};

	initConfirmOrder();
})(window, document, undefined);