(function(window, document, undefined){
	var Promise = function () {
		if (!(this instanceof arguments.callee)) {
			return new arguments.callee();
		}
		
		var state = 'pending',
		message,
		persistent_listeners = [],
		once_listeners = [],
		dfd,
		_this = this;
		
		var setMessage = function (msg) {
			message = msg;
		};
		
		var getMessage = function () {
			return message;
		};
		
		var setState = function (_state) {
			state = _state;
		};
		
		var getState = function () {
			return state;
		};
		
		var isResolved = function () {
			return getState() === 'resolved';
		};
		
		var isRejected = function () {
			return getState() === 'rejected';
		};
		
		var isCompleted = function () {
			return getState() !== 'pending';
		};
		
		var notify2self = function () {
			var fn, callback;
			
			while (callback = once_listeners.shift()) {
				fn = callback[getState()];
				fn(getMessage());
			}
		};
		
		var notify2listener = function () {
			if (dfd) {
				dfd.resolvePromise(_this);
			}
		};
		
		var notify = function () {
			notify2self();
			notify2listener();
		};
		
		this.getMessage = getMessage;
		//this.getState = getState;
		this.isCompleted = isCompleted;
		
		this.setListener = function (_dfd) {
			dfd = _dfd;
		};
		
		this.resolve = function (msg) {
			if (!isCompleted()) {
				setMessage(msg);
				setState('resolved');
				notify();
			}
			return this;
		};
		
		this.resolveAlways = function(msg){
			setMessage(msg);
			setState('resolved');
			notify();
			return this;
		};
		
		this.reject = function (msg) {
			if (!isCompleted()) {
				setMessage(msg);
				setState('rejected');
				notify();
			}
			return this;
		};
		
		this.then = function (success, failure) {
			once_listeners.push({
				'resolved' : success,
				'rejected' : failure
			});
			isCompleted() && notify2self();
			return this;
		};
		
		this.once = function(success, failure){
			this.then(success, failure);
		};
	};

	var NetConnection = function(){
		var stauts = 'disconnected';
		var support = true;
		var self = this;
		
		var setStatus = function(_status){
			status = _status;
		};
		
		var setSupported = function(bool){
			support = bool;
		};
		
		var isSupported = function(){
			return !!support;
		};
		
		var isConnected = function(){
			return status === 'connected';
		};
		
		var isDisconnected = function(){
			return status === 'disconnected';
		};
		
		this.setStatus = setStatus;
		this.isDisconnected = isDisconnected;
		this.isConnected = isConnected;
		
		this.onConnect = (function(){
			var once_listeners4connect = [],
				persistent_listeners4connect = [];
			
			var once_notify = function(){
				var listener;
				while(listener = once_listeners4connect.shift()){
					listener();
				}
			};
			var addOnceListener = function(listener, allowSync){
				if(once_listeners4connect.indexOf(listener) < 0){
					once_listeners4connect.push(listener);
					allowSync && isConnected() && once_notify();
				}
			};
			var notify4connect = function(){
				once_notify();
			};
			
			return {
				addOnceListener: addOnceListener,
				notify: notify4connect
			};
		}());
		
		this.onDisconnect = (function(){
			var once_listeners4disconnect = [],
				persistent_listeners4disconnect = [];
			
			var once_notify = function(){
				var listener;
				while(listener = once_listeners4disconnect.shift()){
					listener();
				}
			};
			var addOnceListener = function(listener, allowSync){
				if(once_listeners4disconnect.indexOf(listener) < 0){
					once_listeners4disconnect.push(listener);
					allowSync && isDisconnected() && once_notify();
				}
			};
			
			var notify = function(){
				var listener;
				for(var i = 0, listener; listener = persistent_listeners4disconnect[i]; i++){
					listener();
				}
			};
			
			var addListener = function(listener, allowSync){
				if(persistent_listeners4disconnect.indexOf(listener) < 0){
					persistent_listeners4disconnect.push(listener);
					allowSync && isDisconnected() && notify();
				}
			};
			
			var notify4disconnect = function(){
				notify();
				once_notify();
			};
			
			return {
				addOnceListener: addOnceListener,
				addListener: addListener,
				notify: notify4disconnect
			};
		}());
		
		var notify = function(){
			isConnected() ? self.onConnect.notify() : self.onDisconnect.notify();
		};
		
		this.notify = notify;
		this.setSupported = setSupported;
		this.isSupported = isSupported;
	};

	var utility = {
		KEY : {
			ESC: 27,
			TAB: 9,
			ENTER: 13
		},
		support: {
			dataset: !!document.body.dataset,
			notification: false
		},
		notifyObj : null,
		timerObj : null,
		notify : function (msg, timeout) {
			this.triggerCustomEvent('sogouExplorerExtensionNotify', {
				message : msg,
				timeout : timeout
			});
		},
		closeNotify : function () {
			this.triggerCustomEvent('sogouExplorerExtensionCloseNotify');
		},
		setPref : function (name, value) {
			window.localStorage.setItem(name, value);
		},
		getPref : function (name) {
			return window.localStorage[name];
		},
		getTimeInfo : function () {
			var d = new Date();
			return d.getHours() + ":" + (d.getMinutes() < 10 ? "0" : "") + d.getMinutes() + ":" + (d.getSeconds() < 10 ? "0" : "") + d.getSeconds();
		},
		savePrefs : function (obj, prefix) {
			var objs = obj.find("input");
			var self = this;
			
			objs.change(function () {
				var type = this.getAttribute("type");
				if (type == "text")
					self.setPref(prefix + "_" + this.getAttribute("id"), $(this).val());
				else if (type == "checkbox")
					self.setPref(prefix + "_" + this.getAttribute("id"), this.checked ? 1 : 0);
			})
		},
		reloadPrefs : function (obj, prefix) {
			var objs = obj.find("input");
			var self = this;
			
			objs.each(function () {
				var e = $(this);
				var type = e.attr("type");
				var id = e.attr("id");
				
				var value = self.getPref(prefix + "_" + id);
				if (typeof(value) == "undefined")
					return;
				
				if (type == "text")
					e.val(value);
				else if (type == "checkbox")
					this.checked = value == "1";
			});
			this.savePrefs(obj, prefix);
		},
		getErrorMsg : function (msg) {
			var m = msg.match(/var\s+message\s*=\s*"([^"]*)/);
			return m && m[1] ? m[1] : "&lt;未知信息&gt;";
		},
		tip: {
			showMask: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionTip.showMask');
			},
			hide: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionTip.hide');
			},
			neterror: {
				popup: function(errorCount, callback){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.neterror', {
						errorCount: errorCount,
						callback: callback
					});
				}
			},
			wait: {
				popup: function(count, callback){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.wait', {
						count: count,
						callback: callback
					});
				}
			},
			noUsername: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noUsername');
				}
			},
			username: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.username');
				}
			},
			noPwd: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noPwd');
				}
			},
			pwd: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.pwd');
				}
			},
			noValidateCode: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noValidateCode');
				}
			},
			validateCode: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.validateCode');
				}
			},
			deny: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.deny');
				}
			},
			
			noFirstTicketTime: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noFirstTicketTime');
				}
			},
			noFirstTicketDate: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noFirstTicketDate');
				}
			},
			
			disconnected: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.disconnected');
				}
			}
		},
		NetConnection: NetConnection,
		getNetConnection: function(){
			var netConnection = new this.NetConnection;
			this.getNetConnection = function(){
				return netConnection;
			};
			return netConnection;
		},
		netConnectionNotify: function(){
			var netConnection = this.getNetConnection();
			netConnection.notify();
		},
		setNetConnectionStatus: function(status){
			var netConnection = this.getNetConnection();
			netConnection.setStatus(status);
		},
		getNetConnectionStatus: function(){
			utility.triggerCustomEvent('sogouExplorerExtensionGetNetConnectionStatus');
		},
		setNetConnectionUnSupported: function(){
			var netConnection = this.getNetConnection();
			netConnection.setSupported(false);
		},
		diffTime: null,
		Promise: Promise,
		getPromise4DiffTime: function(){
			var promise = new this.Promise;
			this.getPromise4DiffTime = function(){
				return promise;
			};
			return promise;
		},
		setDiffTime: function(timestamp){
			utility.diffTime = (+ new Date) - timestamp;
			utility.getPromise4DiffTime().resolve();
		},
		getDiffTime: function(){
			utility.triggerCustomEvent('sogouExplorerExtensionGetRemoteTime');
		},
		queryNotificationSupport: function(){
			utility.triggerCustomEvent('sogouExplorerExtensionNotification.querySupport');
		},
		setNotificationSupport: function(bool){
			utility.support.notification = bool;
		},
		triggerCustomEvent: function(type, data){
			var event;
			if (typeof window.CustomEvent === 'function') {
				event = new window.CustomEvent(type, {
					detail: data ? data : {} 
				});
				document.dispatchEvent(event);
			}else if(typeof document.createEvent === 'function'){
				event = document.createEvent('Event');
				event.initEvent(type);
				
				if(data){
					for(var key in data){
						document.body.dataset[type + '.' + key] = data[key];
					}
				}
				document.dispatchEvent(event);
			}
		},
		pingback: {
			begin: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.begin');
			},
			startlogin: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.startlogin');
			},
			loginsuc: function(autologin_count){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.loginsuc', {count: autologin_count});
			},
			startbuy: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.startbuy');
			},
			buysuc: function(autorefresh_count){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.buysuc', {count: autorefresh_count});
			},
			starttimer: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.starttimer');
			},
			timersuc: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.timersuc');
			}
		}
	};

	var console = window.console || {log: window.alert};
	
	document.addEventListener("sogouExplorerExtensionRemoteTimeReady", function(e){var detail = e.detail || {remoteTime: document.body.dataset["sogouExplorerExtensionRemoteTimeReady.remoteTime"]};var remoteTime = detail.remoteTime;utility.setDiffTime(remoteTime);}, false);
	utility.getDiffTime();
	document.addEventListener("sogouExplorerExtensionNetConnection.change", function(e){var detail = e.detail || {status: document.body.dataset["sogouExplorerExtensionNetConnection.change.status"]};var status = detail.status;utility.setNetConnectionStatus(status);utility.netConnectionNotify();}, false);
	document.addEventListener("sogouExplorerExtensionNetConnection.unsupported", function(e){utility.setNetConnectionUnSupported();}, false);
	utility.getNetConnectionStatus();
	document.addEventListener("sogouExplorerExtensionNotification.querySupportOver", function(e){var detail = e.detail || {support: document.body.dataset["sogouExplorerExtenionNotification.querySupportOver.support"]}; utility.setNotificationSupport(detail.support);}, false);
	utility.queryNotificationSupport();
	
	var initLogin = function() {
		var login_form = $('#loginForm');
		var username_input = $('#UserName');
		var password_input = $('#password');
		var randcode_input = $('#randCode');
		
		var submit_form = function(){
			relogin();
		};
		
		var KEY = utility.KEY;
		
		login_form[0].onsubmit = function(e){
			e.preventDefault();
		};
		login_form.unbind('submit').bind('submit', function(e){
			e.preventDefault();
		});
		username_input.unbind('keyup').bind('keyup', function(e){
			if(e.keyCode === KEY.ENTER){
				e.preventDefault();e.stopPropagation();submit_form();
			}
		});
		password_input.unbind('keyup').bind('keyup', function(e){
			if(e.keyCode === KEY.ENTER){
				e.preventDefault();e.stopPropagation();submit_form();
			}
		});
		randcode_input.unbind('keyup').bind('keyup', function(e){
			if(e.keyCode === KEY.ENTER){
				e.preventDefault();e.stopPropagation();submit_form();
			}
		});

		var auto_login_btn = $('<span id="auto-login-btn">自动登录</span>');
		auto_login_btn.bind('click', relogin).bind('click', function(){
			utility.pingback.startlogin();
		});
		
		var  td= $('#loginForm tr:eq(5) td:eq(1)');
		td.attr('colspan', 2);
		var findPwdInitLink = $('a', td).last();
		findPwdInitLink.css('margin-left', '22px');
		auto_login_btn.insertBefore(findPwdInitLink);
		
		$('#loginForm table tbody').append('<tr style="font-size: 12px; color: gray;"><td colspan="3" style="padding-top: 10px; line-height: normal;"><strong style="color: green;">点击绿色按钮</strong>，搜狗12306助手可帮您自动登录，告别反复登录的烦恼！</td></tr>');

		var count = 0;
		var errorCount = 0;
		
		function getLoginRandCode() {
			$.ajax({
				url : "/otsweb/loginAction.do?method=loginAysnSuggest",
				//method : "POST",
				type : "POST",
				dataType : "json",
				cache : false,
				timeout: 5000,
				success : function (json, code, jqXhr) {
					var netConnection = utility.getNetConnection();
					var supported = netConnection.isSupported();
					var connected = (supported && netConnection.isConnected()) || (!supported && json);
					
					if(connected){
						if (json.randError != 'Y') {
							getLoginRandCode();
						} else {
							$("#loginRand").val(json.loginRand);
							submitForm();
						}
					}else{
						utility.tip.disconnected.popup();
						
						if(supported){
							netConnection.onConnect.addOnceListener(getLoginRandCode, true);
						}
					}
				},
				error : function (xhr) {
					(function(){
						var getLoginRandCode_timer = null;
						
						var callback = function(){
							clearTimeout(getLoginRandCode_timer);
							errorCount = 0;
						};
						var second_param;
						
						if(typeof window.CustomEvent === 'function'){
							second_param = callback;
						}else{
							second_param = 'neterrorPopupCallback';
							document.addEventListener(second_param, function(){
							document.removeEventListener(second_param, arguments.callee, false);
								callback();
							}, false);
						}
						
						errorCount++;
						getLoginRandCode_timer = setTimeout(getLoginRandCode, 2000);
						utility.tip.neterror.popup(errorCount, second_param);
					}());
				}
			});
		}
		
		function submitForm() {
			var data = {};
			
			$.each($("#loginForm").serializeArray(), function () {
				data[this.name] = this.value;
			});
			
			var username = data['loginUser.user_name'],
				pwd = data['user.password'],
				randCode = data['randCode'];
			
			if(/用户名|邮箱/.test(username)){
				utility.tip.noUsername.popup();
				return;
			}else if(!pwd || pwd.length < 6){
				utility.tip.noPwd.popup();
				return;
			}else if(!randCode || randCode.length !== 4){
				utility.tip.noValidateCode.popup();
				return;
			}
			
			utility.setPref("__un", data["loginUser.user_name"]);
			utility.setPref("__up", data["user.password"]);
			
			var try2login = function(){
				$.ajax({
					type : "POST",
					url : "/otsweb/loginAction.do?method=login",
					data : data,
					timeout: 5000,
					dataType : "text",
					success : function (html) {
						var netConnection = utility.getNetConnection();
						
						var supported = netConnection.isSupported();
						var connected = (supported && netConnection.isConnected()) || (!supported && html);
						
						if(connected){
							msg = utility.getErrorMsg(html);
							
							if(/登录名|邮箱/.test(msg)){
								utility.tip.username.popup();
								stopLogin();
							}else if (html.indexOf('请输入正确的验证码') > -1) {
								utility.tip.validateCode.popup();
								stopLogin();
								return;
							} else if (msg.indexOf('密码') > -1) {
								utility.tip.pwd.popup();
								stopLogin();
								return;
							} else if (msg.indexOf('锁定') > -1) {
								utility.tip.deny.popup();
								stopLogin();
								return;
							} else if (html.indexOf("欢迎您") != -1) {
								utility.pingback.loginsuc(count + 1);
								if(utility.support.notification){
									utility.notify('登录成功！');
								}else{
									alert('登陆成功');
								}
								window.location.href = "https://dynamic.12306.cn/otsweb/order/querySingleAction.do?method=init";
								return;
							} else {
								(function(){
									count++;
									auto_login_btn.text(count + '次登录');
									
									var try2login_timer = null;
									
									var callback = function(){
										clearTimeout(try2login_timer);
									};
									var second_param;
									
									if(typeof window.CustomEvent === 'function'){
										second_param = callback;
									}else{
										second_param = 'waitPopupCallback';
										document.addEventListener(second_param, function(){
										document.removeEventListener(second_param, arguments.callee, false);
											callback();
										}, false);
									}
									
									try2login_timer = setTimeout(try2login, 2000);
									utility.tip.wait.popup(count, second_param);
								}());
								
								return;
							}
						}else{
							utility.tip.disconnected.popup();
							if(supported){
								netConnection.onConnect.addOnceListener(try2login, true);
							}
						}
					},
					error : function (msg) {
						(function(){
							var try2login_timer = null;
							
							var callback = function(){
								clearTimeout(try2login_timer);
								errorCount = 0;
							};
							var second_param;
							
							if(typeof window.CustomEvent === 'function'){
								second_param = callback;
							}else{
								second_param = 'neterrorPopupCallback';
								document.addEventListener(second_param, function(){
									document.removeEventListener(second_param, arguments.callee, false);
									callback();
								}, false);
							}
							
							errorCount++;
							try2login_timer = setTimeout(try2login, 2000);
							utility.tip.neterror.popup(errorCount, second_param);
						}());
					}
				});				
			};
			
			try2login();
		}
		
		function relogin() {
			username_input.blur();
			password_input.blur();
			randcode_input.blur();
			utility.tip.showMask();
			getLoginRandCode();
		}
		
		function stopLogin() {
			$("#img_rrand_code").click();
			$("#randCode").val("")[0].select();
		}
		
		var kun = utility.getPref("__un");
		var kup = utility.getPref("__up");
		
		if (kun && kup) {
			$("#UserName").val(kun);
			$("#password").val(kup);
			$("#randCode")[0].focus();
		}
		
		var interval = 1000 * 60 * 1;
		var intervalRequest = function(){
			var requestUrl = 'https://dynamic.12306.cn/otsweb/order/querySingleAction.do?method=init';
			var img = new Image;
			img.src = requestUrl;
			setTimeout(intervalRequest, interval);
		};
		intervalRequest();		
	};
	
	initLogin();
})(window, document, undefined);