(function(window, document, undefined){
	var CountdownTimer = function(args){
		var padding_time,
			handlers = {
				sec: [],
				min: [],
				hour: [],
				end: []
			},
			tick_timer = null;
			
		var init = function(){
			var countdown_time = {
				hour : args.hour || 0,
				min : args.min || 0,
				sec: args.sec || 0
			};
			//var now = new Date, //本地时间
			var now = new Date(Date.now() - utility.diffTime), //服务器时间
				hour = now.getHours(),
				min = now.getMinutes(),
				sec = now.getSeconds();
				
			padding_time = diff(countdown_time, {
				hour: hour,
				min: min,
				sec: sec
			});
		};			
		
		var diff = function(end_time, start_time){
			var hour = end_time.hour - start_time.hour,
				min = end_time.min - start_time.min,
				sec = end_time.sec - start_time.sec,
				time = hour * 3600 + min * 60 + sec,
				ret = false;
				
			if(time >= 0){
				sec = time % 60;
				//min = ((time - sec) % 3600) / 60;
				min = Math.floor((time % 3600) / 60);
				hour = Math.floor(time / 3600);
				
				ret = {
					hour: hour,
					min: min,
					sec: sec
				};
			}
			
			return ret;
		};

		var each = function(items, callback){
			for(var i = 0, item; item = items[i]; i++){
				callback.call(item, item, i);
			}
		};

		var trigger = function(type){
			each(handlers[type], function(handler){
				handler(padding_time[type], withPrefixZero(type));
			});
		};

		var secTick = function(){
			trigger('sec');
		};
		
		var minTick = function(){
			trigger('min');
		};
		
		var hourTick = function(){
			trigger('hour');
		};

		var countdownEnd = function(){
			trigger('end');
		};

		var requestHourTick = function(yes_callback, no_callback){
			var hour = padding_time.hour;
			
			if(hour > 0){
				hour--;
				padding_time.hour = hour;
				hourTick();
				yes_callback && yes_callback();
			}else{
				no_callback && no_callback();
			}
		};
		
		var requestMinTick = function(yes_callback, no_callback){
			var min = padding_time.min;
			
			if(min > 0){
				min--;
				padding_time.min = min;
				minTick();
				yes_callback && yes_callback();
			}else{
				requestHourTick(function(){
					min = 59;
					padding_time.min = min;
					minTick();
					yes_callback && yes_callback();
				}, no_callback);
			}
		};
		
		var countdown = function(){
			tick_timer = setTimeout(function(){
				var sec = padding_time.sec;
				
				if(sec == 0){
					requestMinTick(function(){
						sec = 59;
						padding_time.sec = sec;
						secTick();
						countdown();
					}, function(){
						countdownEnd();
					});
				}else{
					sec--;
					padding_time.sec =sec;
					secTick();
					countdown();
				}
			}, 1000);
		};
		
		this.getHour = function(){
			return padding_time.hour;
		};
		this.getMin = function(){
			return padding_time.min;
		};
		this.getSec = function(){
			return padding_time.sec;
		};
		
		var getHourWithPrefixZero = this.getHourWithPrefixZero = function(){
			var hour = padding_time.hour;
			return hour < 10 ? '0' + hour : hour;
		};
		var getMinWithPrefixZero = this.getMinWithPrefixZero = function(){
			var min = padding_time.min;
			return min < 10 ? '0' + min : min;
		};
		var getSecWithPrefixZero = this.getSecWithPrefixZero = function(){
			var sec = padding_time.sec;
			return sec < 10 ? '0' + sec : sec;
		};
		
		var withPrefixZero = function(type){
			var ret;
			
			if(type === 'sec'){
				ret = getSecWithPrefixZero();
			}else if(type === 'min'){
				ret = getMinWithPrefixZero();
			}else if(type === 'hour'){
				ret = getHourWithPrefixZero();
			}
			
			return ret;
		};
		
		this.addHourHandler = function(fn){
			handlers['hour'].push(fn);
			return this;
		};
		this.addMinHandler = function(fn){
			handlers['min'].push(fn);
			return this;
		};
		this.addSecHandler = function(fn){
			handlers['sec'].push(fn);
			return this;
		};
		this.addEndHandler = function(fn){
			handlers['end'].push(fn);
			return this;
		};
		
		this.start = function(){
			init();
			padding_time && countdown();
			return this;
		};
			
		this.stop = function(){
			clearTimeout(tick_timer);
		};
	};	

	var Promise = function () {
		if (!(this instanceof arguments.callee)) {
			return new arguments.callee();
		}
		
		var state = 'pending',
		message,
		persistent_listeners = [],
		once_listeners = [],
		dfd,
		_this = this;
		
		var setMessage = function (msg) {
			message = msg;
		};
		
		var getMessage = function () {
			return message;
		};
		
		var setState = function (_state) {
			state = _state;
		};
		
		var getState = function () {
			return state;
		};
		
		var isResolved = function () {
			return getState() === 'resolved';
		};
		
		var isRejected = function () {
			return getState() === 'rejected';
		};
		
		var isCompleted = function () {
			return getState() !== 'pending';
		};
		
		var notify2self = function () {
			var fn, callback;
			
			while (callback = once_listeners.shift()) {
				fn = callback[getState()];
				fn(getMessage());
			}
		};
		
		var notify2listener = function () {
			if (dfd) {
				dfd.resolvePromise(_this);
			}
		};
		
		var notify = function () {
			notify2self();
			notify2listener();
		};
		
		this.getMessage = getMessage;
		//this.getState = getState;
		this.isCompleted = isCompleted;
		
		this.setListener = function (_dfd) {
			dfd = _dfd;
		};
		
		this.resolve = function (msg) {
			if (!isCompleted()) {
				setMessage(msg);
				setState('resolved');
				notify();
			}
			return this;
		};
		
		this.resolveAlways = function(msg){
			setMessage(msg);
			setState('resolved');
			notify();
			return this;
		};
		
		this.reject = function (msg) {
			if (!isCompleted()) {
				setMessage(msg);
				setState('rejected');
				notify();
			}
			return this;
		};
		
		this.then = function (success, failure) {
			once_listeners.push({
				'resolved' : success,
				'rejected' : failure
			});
			isCompleted() && notify2self();
			return this;
		};
		
		this.once = function(success, failure){
			this.then(success, failure);
		};
	};

	var NetConnection = function(){
		var stauts = 'disconnected';
		var support = true;
		var self = this;
		
		var setStatus = function(_status){
			status = _status;
		};
		
		var setSupported = function(bool){
			support = bool;
		};
		
		var isSupported = function(){
			return !!support;
		};
		
		var isConnected = function(){
			return status === 'connected';
		};
		
		var isDisconnected = function(){
			return status === 'disconnected';
		};
		
		this.setStatus = setStatus;
		this.isDisconnected = isDisconnected;
		this.isConnected = isConnected;
		
		this.onConnect = (function(){
			var once_listeners4connect = [],
				persistent_listeners4connect = [];
			
			var once_notify = function(){
				var listener;
				while(listener = once_listeners4connect.shift()){
					listener();
				}
			};
			var addOnceListener = function(listener, allowSync){
				if(once_listeners4connect.indexOf(listener) < 0){
					once_listeners4connect.push(listener);
					allowSync && isConnected() && once_notify();
				}
			};
			var notify4connect = function(){
				once_notify();
			};
			
			return {
				addOnceListener: addOnceListener,
				notify: notify4connect
			};
		}());
		
		this.onDisconnect = (function(){
			var once_listeners4disconnect = [],
				persistent_listeners4disconnect = [];
			
			var once_notify = function(){
				var listener;
				while(listener = once_listeners4disconnect.shift()){
					listener();
				}
			};
			var addOnceListener = function(listener, allowSync){
				if(once_listeners4disconnect.indexOf(listener) < 0){
					once_listeners4disconnect.push(listener);
					allowSync && isDisconnected() && once_notify();
				}
			};
			
			var notify = function(){
				var listener;
				for(var i = 0, listener; listener = persistent_listeners4disconnect[i]; i++){
					listener();
				}
			};
			
			var addListener = function(listener, allowSync){
				if(persistent_listeners4disconnect.indexOf(listener) < 0){
					persistent_listeners4disconnect.push(listener);
					allowSync && isDisconnected() && notify();
				}
			};
			
			var notify4disconnect = function(){
				notify();
				once_notify();
			};
			
			return {
				addOnceListener: addOnceListener,
				addListener: addListener,
				notify: notify4disconnect
			};
		}());
		
		var notify = function(){
			isConnected() ? self.onConnect.notify() : self.onDisconnect.notify();
		};
		
		this.notify = notify;
		this.setSupported = setSupported;
		this.isSupported = isSupported;
	};

	var utility = {
		KEY : {
			ESC: 27,
			TAB: 9,
			ENTER: 13
		},
		support: {
			dataset: !!document.body.dataset,
			notification: false
		},
		notifyObj : null,
		timerObj : null,
		notify : function (msg, timeout) {
			this.triggerCustomEvent('sogouExplorerExtensionNotify', {
				message : msg,
				timeout : timeout
			});
		},
		closeNotify : function () {
			this.triggerCustomEvent('sogouExplorerExtensionCloseNotify');
		},
		setPref : function (name, value) {
			window.localStorage.setItem(name, value);
		},
		getPref : function (name) {
			return window.localStorage[name];
		},
		getTimeInfo : function () {
			var d = new Date();
			return d.getHours() + ":" + (d.getMinutes() < 10 ? "0" : "") + d.getMinutes() + ":" + (d.getSeconds() < 10 ? "0" : "") + d.getSeconds();
		},
		savePrefs : function (obj, prefix) {
			var objs = obj.find("input");
			var self = this;
			
			objs.change(function () {
				var type = this.getAttribute("type");
				if (type == "text")
					self.setPref(prefix + "_" + this.getAttribute("id"), $(this).val());
				else if (type == "checkbox")
					self.setPref(prefix + "_" + this.getAttribute("id"), this.checked ? 1 : 0);
			})
		},
		reloadPrefs : function (obj, prefix) {
			var objs = obj.find("input");
			var self = this;
			
			objs.each(function () {
				var e = $(this);
				var type = e.attr("type");
				var id = e.attr("id");
				
				var value = self.getPref(prefix + "_" + id);
				if (typeof(value) == "undefined")
					return;
				
				if (type == "text")
					e.val(value);
				else if (type == "checkbox")
					this.checked = value == "1";
			});
			this.savePrefs(obj, prefix);
		},
		getErrorMsg : function (msg) {
			var m = msg.match(/var\s+message\s*=\s*"([^"]*)/);
			return m && m[1] ? m[1] : "&lt;未知信息&gt;";
		},
		tip: {
			showMask: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionTip.showMask');
			},
			hide: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionTip.hide');
			},
			neterror: {
				popup: function(errorCount, callback){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.neterror', {
						errorCount: errorCount,
						callback: callback
					});
				}
			},
			wait: {
				popup: function(count, callback){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.wait', {
						count: count,
						callback: callback
					});
				}
			},
			noUsername: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noUsername');
				}
			},
			username: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.username');
				}
			},
			noPwd: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noPwd');
				}
			},
			pwd: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.pwd');
				}
			},
			noValidateCode: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noValidateCode');
				}
			},
			validateCode: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.validateCode');
				}
			},
			deny: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.deny');
				}
			},
			
			noFirstTicketTime: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noFirstTicketTime');
				}
			},
			noFirstTicketDate: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.noFirstTicketDate');
				}
			},
			
			disconnected: {
				popup: function(){
					utility.triggerCustomEvent('sogouExplorerExtensionTip.disconnected');
				}
			}
		},
		CountdownTimer: CountdownTimer,
		NetConnection: NetConnection,
		getNetConnection: function(){
			var netConnection = new this.NetConnection;
			this.getNetConnection = function(){
				return netConnection;
			};
			return netConnection;
		},
		netConnectionNotify: function(){
			var netConnection = this.getNetConnection();
			netConnection.notify();
		},
		setNetConnectionStatus: function(status){
			var netConnection = this.getNetConnection();
			netConnection.setStatus(status);
		},
		getNetConnectionStatus: function(){
			utility.triggerCustomEvent('sogouExplorerExtensionGetNetConnectionStatus');
		},
		setNetConnectionUnSupported: function(){
			var netConnection = this.getNetConnection();
			netConnection.setSupported(false);
		},
		diffTime: null,
		Promise: Promise,
		getPromise4DiffTime: function(){
			var promise = new this.Promise;
			this.getPromise4DiffTime = function(){
				return promise;
			};
			return promise;
		},
		setDiffTime: function(timestamp){
			utility.diffTime = (+ new Date) - timestamp;
			utility.getPromise4DiffTime().resolve();
		},
		getDiffTime: function(){
			utility.triggerCustomEvent('sogouExplorerExtensionGetRemoteTime');
		},
		queryNotificationSupport: function(){
			utility.triggerCustomEvent('sogouExplorerExtensionNotification.querySupport');
		},
		setNotificationSupport: function(bool){
			utility.support.notification = bool;
		},
		triggerCustomEvent: function(type, data){
			var event;
			if (typeof window.CustomEvent === 'function') {
				event = new window.CustomEvent(type, {
					detail: data ? data : {} 
				});
				document.dispatchEvent(event);
			}else if(typeof document.createEvent === 'function'){
				event = document.createEvent('Event');
				event.initEvent(type);
				
				if(data){
					for(var key in data){
						document.body.dataset[type + '.' + key] = data[key];
					}
				}
				document.dispatchEvent(event);
			}
		},
		pingback: {
			begin: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.begin');
			},
			startlogin: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.startlogin');
			},
			loginsuc: function(autologin_count){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.loginsuc', {count: autologin_count});
			},
			startbuy: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.startbuy');
			},
			buysuc: function(autorefresh_count){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.buysuc', {count: autorefresh_count});
			},
			starttimer: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.starttimer');
			},
			timersuc: function(){
				utility.triggerCustomEvent('sogouExplorerExtensionPingback.timersuc');
			}
		}
	};

	var console = window.console || {log: window.alert};
	
	document.addEventListener("sogouExplorerExtensionRemoteTimeReady", function(e){var detail = e.detail || {remoteTime: document.body.dataset["sogouExplorerExtensionRemoteTimeReady.remoteTime"]};var remoteTime = detail.remoteTime;utility.setDiffTime(remoteTime);}, false);
	utility.getDiffTime();
	document.addEventListener("sogouExplorerExtensionNetConnection.change", function(e){var detail = e.detail || {status: document.body.dataset["sogouExplorerExtensionNetConnection.change.status"]};var status = detail.status;utility.setNetConnectionStatus(status);utility.netConnectionNotify();}, false);
	document.addEventListener("sogouExplorerExtensionNetConnection.unsupported", function(e){utility.setNetConnectionUnSupported();}, false);
	utility.getNetConnectionStatus();
	document.addEventListener("sogouExplorerExtensionNotification.querySupportOver", function(e){var detail = e.detail || {support: document.body.dataset["sogouExplorerExtenionNotification.querySupportOver.support"]}; utility.setNotificationSupport(detail.support);}, false);
	utility.queryNotificationSupport();
	
	var initTicketQuery = function() {
		var auto_refresh_timer = null;
	
		$('#gridbox .objbox').css('top', function(top){
			return top + 13;
		});
		$('div.gridbox_light table.obj').css('border-bottom', 'none');
		
		var buttonid = "submitQuery";
		//var autoRefresh = false;
		var autoRefresh = true;
		var queryCount = 0;
		var timer = null;
		var isTicketAvailable = false;
		//var audio = null; //通知声音
		//var audioSrc = 'http://www.w3school.com.cn/i/song.ogg';
		var timerCountDown = 0;
		var timeCount = 5;
		var filterNonBookable = false;
		var filterNonNeeded = false;
		var autoBook = false;
		
		utility.reloadPrefs($("tr.append_row"), "ticket_query");
		
		var first_ticket_pingback = false;
		
		var FROM_STATION_TEXT = utility.getPref('_from_station_text'); // 出发站名称
		var FROM_STATION_TELECODE = utility.getPref('_from_station_telecode'); // 出发站电报码
		var TO_STATION_TEXT = utility.getPref('_to_station_text'); // 到达站名称
		var TO_STATION_TELECODE = utility.getPref('_to_station_telecode'); // 到达站电报码
		var DEPART_DATE = utility.getPref('_depart_date'); // 出发日期
		var DEPART_TIME = utility.getPref('_depart_time'); // 出发时间
		
		var is_student_ticket = utility.getPref('_isStudent');
		
		(function(){
			var check_sleep_ticket = $('<span class="lightblue-text check-ticket" id="check-sleep-ticket">只选卧铺</span>');
			var check_seat_ticket = $('<span class="lightblue-text check-ticket" id="check-seat-ticket">只选座票</span>');
			var check_all_ticket = $('<span class="lightblue-text check-ticket" id="check-all-ticket">全部选择</span>');
			var choose_student_ticket_label = $('<label class="blue-text check-ticket"><input type="checkbox" id="choose-student-ticket"/>学生票</label>');
			
			var in_frommid = $('<div class="in_frommid"></div>')
				.append(check_sleep_ticket)
				.append(check_seat_ticket)
				.append(check_all_ticket)
				.append(choose_student_ticket_label)
				.insertAfter('#show1 .in_fromleft')
				.css({
					width: '28%',
					cssFloat: 'left',
					fontSize: '12px',
					marginLeft: '-13%'
				});
			
			$('#show1 .in_fromr').css({width: '20%'});
			
			var addSelected = function($el){
				$el.removeClass('lightblue-text').addClass('blue-text');
			};
			
			var removeSelected = function($el){
				$el.removeClass('blue-text').addClass('lightblue-text');
			};
			
			check_seat_ticket.click(function () {
				removeSelected(check_sleep_ticket);
				removeSelected(check_all_ticket);
				addSelected($(this));
				$(".hdr tr:eq(2) td").each(function (i, e) {
					$(this).find("input").attr("checked", $(this).text().indexOf("座") != -1).change();
				});
			});
			check_sleep_ticket.click(function () {
				removeSelected(check_seat_ticket);
				removeSelected(check_all_ticket);
				addSelected($(this));
				$(".hdr tr:eq(2) td").each(function (i, e) {
					$(this).find("input").attr("checked", $(this).text().indexOf("卧") != -1).change();
				});
			});
			check_all_ticket.click(function(){
				removeSelected(check_seat_ticket);
				removeSelected(check_sleep_ticket);
				addSelected($(this));
				$(".hdr tr:eq(2) td").each(function (i, e) {
					$(this).find("input").attr("checked", true).change();
				});
			});
		}());
		
		
		if (FROM_STATION_TEXT) {
			$("#fromStationText").val(FROM_STATION_TEXT);
			$("#fromStation").val(FROM_STATION_TELECODE);
			$("#toStationText").val(TO_STATION_TEXT);
			$("#toStation").val(TO_STATION_TELECODE);
			//$("#startdatepicker").val(DEPART_DATE);
			$("#startTime").val(DEPART_TIME);
			if(is_student_ticket === 'true' || is_student_ticket == true){
				buttonid = 'stu_submitQuery';
				$('#choose-student-ticket')[0].checked = true;
			}
		}
		
		var first_ticket_btn_status = 'stopped';
		
		//工具栏
		(function(){
			var toolbar = $('<div id="ticket-toolbar"></div>').insertAfter('.box-mid .cx_form');
			
			toolbar.html(''
			+ '<span class="toolbar-mid lightblue-text" id="auto-refresh-text">自动刷票：</span>'
			+ '<span class="toolbar-mid stopped" id="auto-refresh-btn"></span>'
			+ '<span class="toolbar-mid lightblue-text" id="regular-refresh-text">定时抢票：</span>'
			+ '<span class="toolbar-mid" id="choose-tickettime-text">选择出票时间：</span>'
			+ '<select class="toolbar-mid" id="tickettime-options">'
			+ '<option></option>'
			+ '<option class="lightblue-text" value="08">08:00</option>'
			+ '<option class="lightblue-text" value="09">09:00</option>'
			+ '<option class="lightblue-text" value="10">10:00</option>'
			+ '<option class="lightblue-text" value="11">11:00</option>'
			+ '<option class="lightblue-text" value="12">12:00</option>'
			+ '<option class="lightblue-text" value="13">13:00</option>'
			+ '<option class="lightblue-text" value="14">14:00</option>'
			+ '<option class="lightblue-text" value="15">15:00</option>'
			+ '<option class="lightblue-text" value="16">16:00</option>'
			+ '<option class="lightblue-text" value="17">17:00</option>'
			+ '<option class="lightblue-text" value="18">18:00</option>'
			+ '<option class="lightblue-text" value="19">19:00</option>'
			+ '<option class="lightblue-text" value="20">20:00</option>'
			+ '<option class="lightblue-text" value="21">21:00</option>'
			+ '<option class="lightblue-text" value="22">22:00</option>'
			+ '</select>'
			+ '<a href="http://www.12306.cn/mormhweb/zxdt/tlxw_tdbtz53.html" target="_blank" class="toolbar-mid lightblue-text" id="tickettime-link" style="color: rgb(55,125,225);">各地出票时间</a>'
			+ '<span class="toolbar-mid" id="choose-startdate-text">选择出发日期：</span>'	
			+ '<input type="text" class="toolbar-mid" id="toolbar-startdate" readonly/>'
			+ '<span class="toolbar-mid stopped" id="first-ticket-btn"></span>'
			+ '<span id="query-info">尝试次数：<span class="count"></span></span>'
			+ '<span id="first-ticket-info">将在<span class="hour"></span>小时<span class="min"></span>分<span class="sec"></span>秒后自动查询</span>');
			
			var countdown = null;
			var $first_ticket_info = $('#first-ticket-info');
			var $first_ticket_info_sec = $first_ticket_info.find('.sec');
			var $first_ticket_info_min = $first_ticket_info.find('.min');
			var $first_ticket_info_hour = $first_ticket_info.find('.hour');
			
			var $tickettime_options = $('#tickettime-options');
			var $toolbar_startdate = $('#toolbar-startdate');
			var $startdatepicker = $('#startdatepicker');
			
			var getAdultTimeRange = function(){
				var maxOrderDay = 19;
				var maxOrderDay2millisec = maxOrderDay * 24 * 3600 * 1000;
				var minTimestamp = Date.now() - utility.diffTime;
				var maxTimestamp = minTimestamp + maxOrderDay2millisec;
				return [minTimestamp, maxTimestamp];
			};
			
			var getStudentTimeRange = function(){
				var minTimestamp = Date.now() - utility.diffTime;
				var maxTimestamp = (+new Date('2013-3-3'));
				return [minTimestamp, maxTimestamp];
			};
			
			(function(){
				if(DEPART_DATE){
					var time_range = getAdultTimeRange();
					var minTimestamp = time_range[0];
					var maxTimestamp = time_range[1];
				
					var startdateTimestamp = +new Date(DEPART_DATE);
					if(startdateTimestamp < minTimestamp){
						startdateTimestamp = minTimestamp;
					}
					if(startdateTimestamp > maxTimestamp){
						startdataTimestamp = maxTimestamp;
					}
					var startdate = new Date(startdateTimestamp),
						year = startdate.getFullYear(),
						month = startdate.getMonth() + 1,
						month = month < 10 ? '0' + month : month,
						date = startdate.getDate(),
						startdate = '' + year + '-' + month + '-' + (date < 10 ? '0' + date : date);
					
					$startdatepicker.val(startdate);
				}
				
				$startdatepicker[0].setAttribute('readonly', true);
				
				var options = $tickettime_options.find('option');
				
				$tickettime_options.click(function(){
					var ret = options.filter(function(index, option){
						if(option.value){
							var hour = (+option.value) || 0;
							var localCurTime = Date.now();
							var remoteCurTime = localCurTime - utility.diffTime;
							remoteCurTime = new Date(remoteCurTime);
							var remoteCurHour = remoteCurTime.getHours();
							return hour <= remoteCurHour;
						}
						return true;
					});
					$.each(ret, function(index, option){
						option.disabled = true;
						option.className = '';
					});
				});
			}());
		
			var calcDate = function(timestamp){
				var dateTemp = new Date(timestamp);
				var year = dateTemp.getFullYear();
				var month = dateTemp.getMonth() + 1;
				month = month < 10 ? '0' + month : month;
				var date = dateTemp.getDate();
				dateTemp = '' + year + '-' + month + '-' + (date < 10 ? '0' + date : date);
				return dateTemp;
			};
			
			var varyWithTicketType = function(){
				var maxOrderDay,
					maxOrderDay2millisec,
					minTimestamp,
					maxTimestamp,
					minDate,
					maxDate,
					time_range;
					
				if(is_student_ticket === 'true' || is_student_ticket == true){
					time_range = getStudentTimeRange();
				}else{
					time_range = getAdultTimeRange();
				}
				minTimestamp = time_range[0];
				maxTimestamp = time_range[1];
					
				minDate = calcDate(minTimestamp);
				maxDate = calcDate(maxTimestamp);
				
				return [minDate, maxDate];
			};
			
			$toolbar_startdate.focus(function(){
				var date_range = varyWithTicketType();
				var minDate = date_range[0];
				var maxDate = date_range[1];
				WdatePicker({el: this, minDate: minDate, maxDate: maxDate, qsEnabled: false, isShowToday: false});
				$(this).blur();
			});
			
			utility.getPromise4DiffTime().then(function(){
				var date_range = varyWithTicketType();
				var maxDate = date_range[1];
				$toolbar_startdate.val(maxDate);
			});
			
			$('#choose-student-ticket').change(function(){
				is_student_ticket = this.checked;
				
				if(is_student_ticket === 'true' || is_student_ticket == true){
					buttonid = 'stu_submitQuery';
				}else{
					buttonid = 'submitQuery';
				}
				
				var date_range = varyWithTicketType();
				var maxDate = date_range[1];
				$toolbar_startdate.val(maxDate);
			});
			
			var showFirstTicketInfo = function(hour, min, sec){
				$first_ticket_info_hour.text(hour);
				$first_ticket_info_min.text(min);
				$first_ticket_info_sec.text(sec);
				$first_ticket_info.show();
			};
			
			var hideFirstTicketInfo = function(){
				$first_ticket_info.hide();
			};
			
			$('#first-ticket-btn').click(function(){
				if(first_ticket_btn_status === 'running'){
					first_ticket_btn_status = 'stopped';
					$(this).removeClass('running').addClass('stopped');
					countdown && countdown.stop && countdown.stop();
					countdown = null;
					hideFirstTicketInfo();
					$tickettime_options.attr('disabled', false);
					$toolbar_startdate.attr('disabled', false);
					
					if(auto_refresh_btn_status === 'stopped'){
						$('#choose-student-ticket')[0].disabled = false;
					}
				}else{
					var fromStation4FirstTicket = $('#fromStation').val();
					var toStation4FirstTicket = $('#toStation').val();
					var fromStationText4FirstTicket = $('#fromStationText').val();
					var toStationText4FirstTicket = $('#toStationText').val();
					
					var startdate4FirstTicket = $toolbar_startdate.val();
					var starttime4FirstTicket = $tickettime_options.val();
					
					if(!fromStation4FirstTicket){
						//$('#submitQuery').click();
						$('#' + buttonid).click();
					}else if(!toStation4FirstTicket){
						//$('#submitQuery').click();
						$('#' + buttonid).click();
					}else if(fromStation4FirstTicket == toStation4FirstTicket){
						//$('#submitQuery').click();
						$('#' + buttonid).click();
					}else if(!starttime4FirstTicket){
						alert('请填写出票时间！');
					}else if(!startdate4FirstTicket){
						alert('请填写出发日期！');
					}else{
						first_ticket_btn_status = 'running';
						utility.pingback.starttimer();
						$(this).removeClass('stopped').addClass('running');
						$('#choose-student-ticket')[0].disabled = true;
						countdown = new utility.CountdownTimer({
							hour: starttime4FirstTicket
						});
						
						countdown
							.addSecHandler(function(sec, secWithPrefixZero){
								$first_ticket_info_sec.text(secWithPrefixZero);
							})
							.addMinHandler(function(min, minWithPrefixZero){
								$first_ticket_info_min.text(minWithPrefixZero);
							})
							.addHourHandler(function(hour, hourWithPrefixZero){
								$first_ticket_info_hour.text(hourWithPrefixZero);
							})
							.addEndHandler(function(){
								first_ticket_btn_status = 'stopped';
								first_ticket_pingback = true;
								
								$('#first-ticket-btn').removeClass('running').addClass('stopped');
								countdown = null;
								hideFirstTicketInfo();
								
								$tickettime_options.attr('disabled', false);
								$tickettime_options.val('').change();
								$toolbar_startdate.attr('disabled', false);
								
								if(auto_refresh_btn_status === 'stopped'){
									$('#choose-student-ticket')[0].disabled = false;
								}
								
								$('#fromStationText').val(fromStationText4FirstTicket);
								$('#fromStation').val(fromStation4FirstTicket);
								$('#toStationText').val(toStationText4FirstTicket);
								$('#toStation').val(toStation4FirstTicket);
								$('#startdatepicker').val(startdate4FirstTicket);
								
								if(auto_refresh_btn_status === 'stopped'){
									$('#auto-refresh-btn').click();
								}
							})
							.start();
						
						showFirstTicketInfo(countdown.getHourWithPrefixZero(), countdown.getMinWithPrefixZero(), countdown.getSecWithPrefixZero());
						
						//disable the select
						$tickettime_options.attr('disabled', true);
						
						//disable the input
						$toolbar_startdate.attr('disabled', true);
					}
				}
			});
		}());

		
		//保存信息
		function saveStateInfo() {
			utility.setPref("_from_station_text", $("#fromStationText").val());
			utility.setPref("_from_station_telecode", $("#fromStation").val());
			utility.setPref("_to_station_text", $("#toStationText").val());
			utility.setPref("_to_station_telecode", $("#toStation").val());
			utility.setPref("_depart_date", $("#startdatepicker").val());
			utility.setPref("_depart_time", $("#startTime").val());
			utility.setPref('_isStudent', $('#choose-student-ticket')[0].checked);
		}
		
		
		var autoRefreshBtnStatus2running = function(){
			auto_refresh_btn_status = 'running';
			$('#choose-student-ticket')[0].disabled = true;
			$('#auto-refresh-btn').removeClass('stopped').addClass('running');
			displayQueryInfo();
		};
		
		var autoRefreshBtnStatus2stopped = function(){
			auto_refresh_btn_status = 'stopped';
			if(first_ticket_btn_status === 'stopped'){
				$('#choose-student-ticket')[0].disabled = false;
			}
			$('#auto-refresh-btn').removeClass('running').addClass('stopped');
			hideQueryInfo();
		};
		
		function toggleAutoRefreshBtnStatus() {
			if(auto_refresh_btn_status === 'running'){
				autoRefreshBtnStatus2stopped();
			} else {
				autoRefreshBtnStatus2running();
			}
		}
		
		var auto_refresh_btn_status = 'stopped';
		// 0 代表未发送，1代表已发送，2代表已返回
		var STATUS_NOT_SEND = 0;
		var STATUS_SEND = 1;
		var STATUS_RECEIVED = 2;
		var first_request_status = STATUS_NOT_SEND;
		
		$('#auto-refresh-btn').click(function(){
			if(auto_refresh_btn_status === 'running'){
				toggleAutoRefreshBtnStatus();
				resetTimer();
			}else{
				if($('#submitQuery')[0].disabled || $('#stu_submitQuery')[0].disabled){
				}else{
					var fromStation = $('#fromStation').val();
					var toStation = $('#toStation').val();
					
					var startdatepickerval = $('#startdatepicker').val();
					
					if(!fromStation){
						//$('#submitQuery').click();
						$("#" + buttonid).click();
						//canquery();
					}else if(!toStation){
						//$('#submitQuery').click();
						$("#" + buttonid).click();
						//canquery();
					}else if(fromStation == toStation){
						//$('#submitQuery').click();
						$("#" + buttonid).click();
						//canquery();
					}else if(!startdatepickerval){
						//$('#submitQuery').click();
						$("#" + buttonid).click();
						//canquery();
					}else{
						first_request_status = STATUS_SEND;
						// 这里不改变按钮状态，等到ajaxComplete以后再改变
						utility.pingback.startbuy();
						//$('#submitQuery').click();
						//if(canquery()){
							//$("#" + buttonid).click();
						//}
						
						$("#" + buttonid).click();
					}
				}
			}
		});

		//var queryevent = function () {
			//buttonid = this.getAttribute("id");
		//};
		
		$("#submitQuery, #stu_submitQuery").click(saveStateInfo);
		//.click(queryevent);
		
		function resetTimer() {
			queryCount = 0;
			
			if (timer) {
				clearTimeout(timer);
				timer = null;
			}
		}
		
		function countDownTimer() {
			timerCountDown--;
			
			if (timerCountDown > 0) {
				timer = setTimeout(countDownTimer, 1000);
			} else {
				timer = null;
				doQuery();
			}
		}
		
		var hideQueryInfo = function(){
			$('#query-info').hide();
		};
		
		function displayQueryInfo() {
			queryCount++;
			$('#query-info .count').text(queryCount);
			$('#query-info').show();
		}
		
		function doQuery() {
			if(auto_refresh_btn_status === 'running'){
				displayQueryInfo();
			}
			
			if(canquery()){
				var disabled = $('#stu_submitQuery')[0].disabled;
				
				if(disabled === 'true' || disabled == true){
					/*auto_refresh_timer = setTimeout(function(){
						$("#" + buttonid).click();
					}, timeCount);*/
					$("#" + buttonid).click();
				}else{
					$("#" + buttonid).click();
				}
			}else{
				autoRefreshBtnStatus2stopped();
			}
		}
		
		var highLightRow = function (row) {
			row.css("background-color", "#FD855C");
		}
		var highLightCell = function (cell) {
			cell.css("background-color", "#95AFFD");
		}
		var onticketAvailable = function () {
			if(auto_refresh_btn_status === 'running'){
				toggleAutoRefreshBtnStatus();
				utility.pingback.buysuc(queryCount);
			}
			
			if(first_ticket_pingback){
				first_ticket_pingback = false;
				utility.pingback.timersuc();
			}
			resetTimer();
			hideQueryInfo();
			setTimeout(function () {
				if(utility.support.notification){
					utility.notify('可以买票了！');
				}else{
					alert('可以买票了！');
				}
			}, 100);
		}
		
		var ticketType = new Array();
		
		$(".hdr tr:eq(2) td").each(function (i, e) {
			ticketType.push(false);
			if (i < 3)
				return;
			ticketType[i] = true;
			
			var c = $("<input/>").attr("type", "checkBox").attr("checked", true);
			c[0].ticketTypeId = i;
			c.change(
				function () {
				ticketType[this.ticketTypeId] = this.checked;
			}).appendTo(e);
		});
		
		var checkTickets = function (row) {
			var hasTicket = 1;
			if ($("input.yuding_x", row).length > 0)
				return 0;
			
			$("td", row).each(function (i, e) {
				if (!ticketType[i - 1])
					return;
				var el = $(e);
				
				var info = $.trim(el.text());
				//if (info != "--" && info != "无") {
				if(info === '有' || /\d+/.test(info)){
					hasTicket = 2;
					highLightCell(el);
				}else if(/\d+点起售/.test(info) ){
					resetTimer();
				}
			});
			
			return hasTicket;
		}
		
		$("body").ajaxComplete(function (e, r, s) {
			if (s.url.indexOf("queryLeftTicket") == -1)
				return;
			
			var rows = $("table.obj tr:gt(0)");
			var ticketValid = false;
			
			rows.each(function () {
				var row = $(this);
				var valid = checkTickets($(this));
				if (valid == 2)
					highLightRow(row);
				else {
					if (valid == 1 && filterNonNeeded)
						row.hide();
					if (valid == 0 && filterNonBookable)
						row.hide();
				}
				ticketValid = ticketValid || valid == 2;
			});
			
			if (first_request_status === STATUS_SEND) {
				first_request_status = STATUS_RECEIVED;
				if (auto_refresh_btn_status === 'stopped') {
					toggleAutoRefreshBtnStatus();
				}
			}
			
			if(ticketValid){
				onticketAvailable();
			}else{
				if(auto_refresh_btn_status === 'running'){
					timerCountDown = timeCount + 1;
					countDownTimer();
				}
			}
		});
		
		$("body").ajaxError(function (e, r, s) {
			if (s.url.indexOf("/otsweb/order/querySingleAction.do") != -1) {
				$("#" + $("#queryid").val()).click();
			}
		});
			
		//$.ajaxSetup({cache: ajaxCache});
		
		window.invalidQueryButton = function () {
			var queryButton = $("#submitQuery");
			queryButton.unbind("click", sendQueryFunc);
			if (queryButton.attr("class") == "research_u") {
				renameButton("research_x");
			} else if (queryButton.attr("class") == "search_u") {
				renameButton("search_x");
			}
		};
		
		var intervalRequest = function(interval){
			interval = interval || 1000 * 60 * 1;
			var requestUrl,
				img = new Image;
			
			requestUrl = '/otsweb/order/querySingleAction.do?method=init&t=' + Date.now();
			img.src = requestUrl;
			
			requestUrl = '/otsweb/order/querySingleAction.do?method=queryLeftTicket';
			img.src = requestUrl;
			
			setTimeout(intervalRequest, interval);
		};
		intervalRequest();
	};
	
	initTicketQuery();
})(window, document, undefined);